package com.lstznkj.groundstation.DTO.crane_station;

import com.lstznkj.groundstation.model.crane.DataSource;
import com.lstznkj.groundstation.model.crane.DataType;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * A DTO for the {@link com.lstznkj.groundstation.model.crane.DataType} entity
 */
public class DataTypeDTO implements Serializable {
    private final Integer id;
    private final String name;
    private final int valueType;
    private final int method;
    private final String expression;
    private final List<DataSourceDTO> dataSources;
    private final Timestamp createTimestamp;
    private final Timestamp updateTimestamp;
    private final String remark;

    private int value;
    private String label;
    private List<Child> children=new ArrayList();

    class Child{
        int value;
        String label;

        public Child(int value, String label) {
            this.value = value;
            this.label = label;
        }

        public int getValue() {
            return value;
        }

        public void setValue(int value) {
            this.value = value;
        }



        public String getLabel() {
            return label;
        }

        public void setLabel(String label) {
            this.label = label;
        }
    }

    public DataTypeDTO(Integer id, String name, int valueType, int method, String expression, List<DataSourceDTO> dataSources, Timestamp createTimestamp, Timestamp updateTimestamp, String remark) {
        this.id = id;
        this.name = name;
        this.valueType = valueType;
        this.method = method;
        this.expression = expression;
        this.dataSources = dataSources;
        this.createTimestamp = createTimestamp;
        this.updateTimestamp = updateTimestamp;
        this.remark = remark;
        this.value=id;
        this.label=name;
    }
  public DataTypeDTO(DataType dataType){
      this.id = dataType.getId();
      this.name = dataType.getName();
      this.valueType = dataType.getValueType();
      this.method = dataType.getMethod();
      this.expression = dataType.getExpression();
      this.dataSources=new ArrayList<>();
      for(DataSource ds: dataType.getDataSources()){
          this.dataSources.add(new DataSourceDTO(ds));
          this.children.add(new Child(ds.getId(),ds.getDeviceInfo()+ds.getName()));
      }
      this.createTimestamp = dataType.getCreateTimestamp();
      this.updateTimestamp = dataType.getUpdateTimestamp();
      this.remark = dataType.getRemark();
      this.value=dataType.getId();
      this.label=dataType.getName();
  }

    public Integer getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public int getValueType() {
        return valueType;
    }

    public int getMethod() {
        return method;
    }

    public int getValue() {
        return value;
    }

    public void setValue(int value) {
        this.value = value;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public List<Child> getChildren() {
        return children;
    }

    public void setChildren(List<Child> children) {
        this.children = children;
    }

    public String getExpression() {
        return expression;
    }

    public List<DataSourceDTO> getDataSources() {
        return dataSources;
    }

    public Timestamp getCreateTimestamp() {
        return createTimestamp;
    }

    public Timestamp getUpdateTimestamp() {
        return updateTimestamp;
    }

    public String getRemark() {
        return remark;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        DataTypeDTO entity = (DataTypeDTO) o;
        return Objects.equals(this.id, entity.id) &&
                Objects.equals(this.name, entity.name) &&
                Objects.equals(this.valueType, entity.valueType) &&
                Objects.equals(this.method, entity.method) &&
                Objects.equals(this.expression, entity.expression) &&
                Objects.equals(this.dataSources, entity.dataSources) &&
                Objects.equals(this.createTimestamp, entity.createTimestamp) &&
                Objects.equals(this.updateTimestamp, entity.updateTimestamp) &&
                Objects.equals(this.remark, entity.remark);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, name, valueType, method, expression, dataSources, createTimestamp, updateTimestamp, remark);
    }

    @Override
    public String toString() {
        return getClass().getSimpleName() + "(" +
                "id = " + id + ", " +
                "name = " + name + ", " +
                "valueType = " + valueType + ", " +
                "method = " + method + ", " +
                "expression = " + expression + ", " +
                "dataSources = " + dataSources + ", " +
                "createTimestamp = " + createTimestamp + ", " +
                "updateTimestamp = " + updateTimestamp + ", " +
                "remark = " + remark + ")";
    }
}