package com.lstznkj.groundstation;

import com.lstznkj.groundstation.model.*;
import com.lstznkj.groundstation.repo.AuthorityRepository;
import com.lstznkj.groundstation.repo.SysRoleRepository;
import com.lstznkj.groundstation.repo.UserRepository;
import com.lstznkj.groundstation.service.AuthorityService;
import com.lstznkj.groundstation.service.InitService;
import com.lstznkj.groundstation.service.SysRoleService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.ViewControllerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import java.util.ArrayList;
import java.util.List;

@Configuration
public class MVCConfig implements WebMvcConfigurer {
    private Logger logger = LoggerFactory.getLogger(MVCConfig.class);
    @Autowired
    private UserRepository userRepository;
    @Autowired
    private AuthorityService authorityService;

    @Autowired
    private AuthorityRepository authorityRepository;

    @Autowired
    private SysRoleRepository sysRoleRepository;
    @Autowired
    private SysRoleService sysRoleService;
    @Autowired
    private BCryptPasswordEncoder passwordEncoder;
    @Autowired
    private InitService initService;

    public void addViewControllers(ViewControllerRegistry registry) {
        registry.addViewController("/home").setViewName("home");

        registry.addViewController("/hello").setViewName("hello");
        registry.addViewController("/login").setViewName("login");

        initAuthority();
        // initSysRole();

        initService.initState();
        initService.initDataTypes();
        // initService.initAuthorities();
        initService.initSysRoles();
        initService.initUsers();

        // initService.initTestCraneData();

        initTestData();

    }

    private void initAuthority() {
        // 后端添加权限-权限
        Authority authAuthAdd = new Authority();
        // 后端删除权限-权限
        Authority authAuthDel = new Authority();
        // 后端修改权限-权限
        Authority authAuthUpd = new Authority();
        // 后端查看权限-权限
        Authority authAuthGet = new Authority();

        Authority systemSettingAuth = new Authority();
        systemSettingAuth.setName("系统设置菜单");

    }

    private void initSysRole() {

        // 终极管理员，拥有修改删除所有信息权限
        SysRole ultimateSysRole = new SysRole();
        // 普同用户
        SysRole userSysRole = new SysRole();
        // 管理员
        SysRole adminSysRole = new SysRole();

        adminSysRole.setName("ROLE_ADMIN");
        adminSysRole.setDescription("普通管理员");

        userSysRole.setName("ROLE_USER");
        userSysRole.setDescription("普通用户");

        ultimateSysRole.setName(("ROLE_ULTIMATE"));
        ultimateSysRole.setDescription("超级管理员");

        //// 初始化普通用户角色
        if (!sysRoleRepository.existsSysRoleByName(userSysRole.getName())) {
            logger.info("user role has not been existed");
            Authority getUserInfoAuthority = authorityRepository.findAuthorityByName("获取用户信息");
            userSysRole.getAuthorities().add(getUserInfoAuthority);
            getUserInfoAuthority.getSysRoles().add(userSysRole);

            userSysRole = sysRoleRepository.save(userSysRole);

            sysRoleRepository.save(userSysRole);

        } else {

            logger.info("user role has been existed");

        }
        // end-初始化普通用户角色

        // 初始化超级管理员角色
        if (!sysRoleRepository.existsSysRoleByName(ultimateSysRole.getName())) {
            logger.info("ultimate role has not been existed");
            // 设置权限

            List<SysRole> sysRoles = new ArrayList<>();
            sysRoles.add(ultimateSysRole);
            List<Authority> authorityList = new ArrayList<>();

            // 添加增删查改角色 权限
            // authorityList.addAll(authorityRepository.findAuthoritiesByApiUrl("/roles/**"));
            authorityList.addAll(authorityRepository.findAuthoritiesByApiUrl("/**"));

            ultimateSysRole = sysRoleRepository.save(ultimateSysRole);

            ultimateSysRole.setAuthorities(authorityList);

            for (Authority auth : authorityList) {
                auth.getSysRoles().add(ultimateSysRole);
                // authorityRepository.save(auth);
            }

            sysRoleRepository.save(ultimateSysRole);

        } else {
            logger.info("ultimate role has  been existed");

        }
        // end-初始化超级管理员角色

    }

    /****
     * 初始化用户数据，用户与角色 多对多关系 存在bug，没法通过角色获取用户
     */
    private void initTestData() {

        User adminUlt = new User();
        SysRole adminSysRole = sysRoleService.getAdminSysRole();

        adminUlt.setName("ultimate");
        adminUlt.setUsername("adminult");
        adminUlt.setPassword(passwordEncoder.encode("123456"));

        if (!userRepository.existsUserByUsername(adminUlt.getUsername())) {
            adminUlt = userRepository.save(adminUlt);

            List<SysRole> roles = new ArrayList<>();
            roles.add(adminSysRole);
            SysRole ultimateSysRole = sysRoleService.getUltimateSysRole();
            ultimateSysRole.getUsers().add(adminUlt);
            sysRoleRepository.save(ultimateSysRole);

            roles.add(ultimateSysRole);

            adminUlt.setSysRoles(roles);

            userRepository.save(adminUlt);
        }

    }

    @Override
    public void addCorsMappings(CorsRegistry registry) {
        registry.addMapping("/**")
                .allowCredentials(true)
                .allowedOrigins("*")
                .allowedMethods("PUT", "POST", "GET", "DELETE").exposedHeaders("Authorization");

        registry.addMapping("/users/**")
                .allowCredentials(true)
                .allowedOrigins("*")
                .allowedMethods("PUT", "POST", "GET", "DELETE").exposedHeaders("Authorization");

    }

}