package com.lstznkj.groundstation.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.lstznkj.groundstation.DTO.ui.DeviceDTO;
import com.lstznkj.groundstation.model.crane.Device;
import com.lstznkj.groundstation.security.jwt.JWTUtils;
import com.lstznkj.groundstation.service.crane.DataSourceService;
import com.lstznkj.groundstation.service.crane.DeviceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

@RestController
public class DeviceController {
    @Autowired
    private DeviceService deviceService;
    @Autowired
    private DataSourceService dataSourceService;
    private Logger logger= LoggerFactory.getLogger(getClass());
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private JWTUtils jwtUtils;
    @Deprecated
    @RequestMapping(value = "/devices",method = RequestMethod.POST)
    public Object addDevice(HttpServletRequest request,@RequestBody DeviceDTO deviceDTO){
        String token = request.getHeader("Authorization");
        String realUsername=jwtUtils.getUsername(token);

        try {
            logger.info(realUsername+" start to add device:"+objectMapper.writeValueAsString(deviceDTO));
        }
        catch (Exception ex){
            ex.printStackTrace();
        }
        return deviceService.add(deviceDTO);
    }
    @RequestMapping(value = "/devices/{id}",method = RequestMethod.PUT)
    public Object updateDevice(HttpServletRequest request,@PathVariable int id, @RequestBody DeviceDTO deviceDTO){
        String token = request.getHeader("Authorization");
        String realUsername=jwtUtils.getUsername(token);

        try {
            logger.info(realUsername+" start to update device:"+objectMapper.writeValueAsString(deviceDTO));
            return deviceService.updateDevice(deviceDTO);
        }
        catch (Exception ex){
            ex.printStackTrace();
        }
        return null;
    }
    @RequestMapping(value = "/devices/{id}",method = RequestMethod.DELETE)
    public Object deleteDevice(HttpServletRequest request,@PathVariable int id){
        String token = request.getHeader("Authorization");
        String realUsername=jwtUtils.getUsername(token);

        try {
            logger.info(realUsername+" start to delete device:"+id);

        }
        catch (Exception ex){
            ex.printStackTrace();
        }
        return deviceService.deleteById(id);
    }

    @RequestMapping(value = "/devices/{id}/children",method = RequestMethod.GET)
    public Object getChildrenDevice(@PathVariable int id){

        return deviceService.getChildren(id);
    }
    @RequestMapping(value = "/devices/{id}",method = RequestMethod.GET)
    public Object getDevice(@PathVariable int id){

        return deviceService.getChildren(id);
    }
    @RequestMapping(value = "/devices",method = RequestMethod.GET)
    public Object getAll(){

        return deviceService.getAll();
    }

    /***
     *获取天车列表
     * @return
     */
    @RequestMapping(value = "/devices/level/0",method = RequestMethod.GET)
    public Object getRootDevices(){
        return deviceService.getRootDevices();
    }

    /***
     *
     * 获取指定id设备 下所有的 子设备数据源
     * @param id
     * @return
     */

    @RequestMapping(value = "/devices/{id}/childrenDataSources",method = RequestMethod.GET)
    public Object getDataSourcesByDeviceId(HttpServletRequest request,@PathVariable int id) throws JsonProcessingException {
        String token = request.getHeader("Authorization");
        String realUsername=jwtUtils.getUsername(token);

        logger.info(realUsername+" start to get all childrenDatasources");
      //  Device device= deviceService.getDeviceById(id).getData();
//        if(device==null){
//            logger.info("device "+ id+" is null");
//            return null;
//        }
//        else{
//            //logger.info("device "+id+":"+objectMapper.writeValueAsString(device));
//            return device.getChildrenDataSources();
//        }
        return dataSourceService.getAllDataSources(id);

    }

    /**
     * 获取指定设备的某个数据源的历史检测数据
     *
     */
    @RequestMapping(value = "/devices/{deviceId}/dataSourceUIDTOs/{sourceId}/{startTime}/{endTime}",method = RequestMethod.GET)
    public Object getDeviceDataSourceDTO(@PathVariable int deviceId,@PathVariable int sourceId,@PathVariable String startTime,@PathVariable String endTime){

        return  dataSourceService.getDataSourceUIDTO(sourceId,startTime,endTime);
    }

    /**
     * 获取指定设备的所有数据源(包括子设备)
     * @param id
     * @return
     */
//    @RequestMapping(value = "/devices/{id}/dataSources",method = RequestMethod.GET)
//    public Object getRootDeviceDataSourceList(@PathVariable int id){
//
//        return dataSourceService.getAllDataSources(id);
//
//    }

    /**
     * 获取指定设备的所有数据源(包括子设备)
     * @param id
     * @return
     */
    @RequestMapping(value = "/devices/{id}/dataSourcePoolDTO",method = RequestMethod.GET)
    public Object getRootDeviceDataSourceDTOList(@PathVariable int id){

        return dataSourceService.getDataSourcePoolDTO(id);
    }



}
