package com.lstznkj.groundstation.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.lstznkj.groundstation.DTO.ui.EquipmentDTO;
import com.lstznkj.groundstation.model.Result;
import com.lstznkj.groundstation.model.crane.extra.Equipment;
import com.lstznkj.groundstation.service.ImageService;
import com.lstznkj.groundstation.service.crane.extra.EquipmentService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

@RestController
public class EquipmentController {
    private Logger logger= LoggerFactory.getLogger(getClass());
    // @Value("${demo.imgPath}")
    // private String imgPath;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private EquipmentService equipmentService;
    @Autowired
    private ImageService imageService;
    @RequestMapping(method = RequestMethod.POST,value = "/equipments")
    public Object add(@RequestBody EquipmentDTO equipmentDTO){

        try {
            logger.info("start to add equipment:"+objectMapper.writeValueAsString(equipmentDTO));
            return  equipmentService.add(equipmentDTO);
        }
        catch (Exception ex){
            ex.printStackTrace();
        }

        return null;
    }
    @RequestMapping(method = RequestMethod.PUT,value = "/equipments/{id}")
    public Object update(@PathVariable int id, @RequestBody EquipmentDTO equipmentDTO){

        try {
            logger.info("start to update equipment:"+objectMapper.writeValueAsString(equipmentDTO));
            return  equipmentService.update(id,equipmentDTO);
        }
        catch (Exception ex){
            ex.printStackTrace();
        }

        return null;
    }

    @RequestMapping(method = RequestMethod.DELETE,value = "/equipments/{id}")
    public Object delete(@PathVariable int id, @RequestBody EquipmentDTO equipmentDTO){

        try {
            logger.info("start to delete equipment:"+objectMapper.writeValueAsString(equipmentDTO));
            return  equipmentService.deleteById(id);
        }
        catch (Exception ex){
            ex.printStackTrace();
        }

        return null;
    }
    @RequestMapping(method = RequestMethod.GET,value = "/equipments")
    public Object getAll(){

        try {

            logger.info("start to get all equipment:");

            return equipmentService.getAll();
        }
        catch (Exception ex){
            ex.printStackTrace();
        }

        return null;
    }
    @RequestMapping(method = RequestMethod.GET,value = "/cranes/{id}/equipments")
    public Object getAllEquipmentsForCrane(@PathVariable int id){

        try {

            logger.info("start to get all equipment for crane:"+id);

            return equipmentService.getAllEquipmentsForCrane(id);
        }
        catch (Exception ex){
            ex.printStackTrace();
        }

        return null;
    }

    /**
     * 文件上传
     * @param img
     * @param request
     * @return
     */
    @RequestMapping(value = "/equipments/{uuid}/img",method = RequestMethod.POST)
    public Object upload(@RequestParam(value = "img") MultipartFile img,  @PathVariable String uuid, HttpServletRequest request) {

       try {
           return imageService.uploadImage(img,uuid);
       }catch (Exception ex){
           ex.printStackTrace();
       }

       return null;

    }

    /***
     * 获取指定uuid的照片
     * @param uuid
     * @return

     */
    @RequestMapping(value = "/equipments/{uuid}/img",method = RequestMethod.GET)
    public Object getImg(@PathVariable String uuid)  {

        try {
            logger.info("start to get image by uuid:"+uuid);
            return imageService.getImageByUuid(uuid);
        }
        catch (Exception ex){
            ex.printStackTrace();
        }
        return null;
    }

}
