package com.lstznkj.groundstation.controller;

import com.lstznkj.groundstation.model.Authority;
import com.lstznkj.groundstation.model.SysRole;
import com.lstznkj.groundstation.security.jwt.JWTUtils;
import com.lstznkj.groundstation.service.AuthorityService;
import com.lstznkj.groundstation.service.SysRoleService;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.swagger.annotations.Api;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

@Api("超级管理员")
@RestController
@RequestMapping(value = "/ultimate")
public class UltimateController {

    private Logger logger= LoggerFactory.getLogger(UltimateController.class);
    @Autowired
    private JWTUtils jwtUtils;
    @Autowired
    private ObjectMapper objectMapper;
  
    @Autowired
    private AuthorityService authorityService;
    @Autowired
    private SysRoleService sysRoleService;

    @RequestMapping(value = "/roles",method = RequestMethod.GET)
    public Object getAllRoles(HttpServletRequest request)
    {
        String token = request.getHeader("Authorization");
        logger.info(jwtUtils.getUsername(token)+" start to get roles");

        return  sysRoleService.findAll();

    }
    @RequestMapping(value = "/roles/{id}",method = RequestMethod.PUT)
    public Object updateRole(HttpServletRequest request, @RequestBody SysRole role,@PathVariable int id)
    {
        String token = request.getHeader("Authorization");
        try {
            logger.info(jwtUtils.getUsername(token)+" start to update role"+objectMapper.writeValueAsString(role));
        }
        catch (Exception ex)
        {
            logger.error(jwtUtils.getUsername(token)+" start to update role failed!");

        }

        return sysRoleService.update(role);

    }
    @RequestMapping(value = "/roles/{id}",method = RequestMethod.DELETE)
    public Object deleteRole(HttpServletRequest request, @PathVariable int id)
    {
        String token = request.getHeader("Authorization");
        try {
            logger.info(jwtUtils.getUsername(token)+" start to delete role"+id);
        }
        catch (Exception ex)
        {
            logger.error(jwtUtils.getUsername(token)+" deleted role failed!");

        }

        return sysRoleService.deleteById(id);

    }

    @RequestMapping(value = "/roles",method = RequestMethod.POST)
    public Object addRole(HttpServletRequest request, @RequestBody SysRole role)
    {
        String token = request.getHeader("Authorization");
        try {
            logger.info(jwtUtils.getUsername(token)+" start to add a  role:"+objectMapper.writeValueAsString(role));
        }
        catch (Exception ex)
        {
            logger.error(jwtUtils.getUsername(token)+" added role failed!");

        }

        return sysRoleService.save(role);

    }


    @RequestMapping(value = "/routers",method = RequestMethod.GET)
    public  Object getAllRouters(HttpServletRequest request)
    {
        String token = request.getHeader("Authorization");

        logger.info(jwtUtils.getUsername(token)+" start to get router");

        return "routers";
    }

   @RequestMapping(value = "/authorities",method = RequestMethod.GET)
    public  Object getAllAuthorities(HttpServletRequest request)
    {
        String token = request.getHeader("Authorization");

        logger.info(jwtUtils.getUsername(token)+" start to get authorities");

        return authorityService.findAll(token);
    }
    @RequestMapping(value = "/authorities",method = RequestMethod.POST)
    public Object addAuthority(HttpServletRequest request, @RequestBody Authority authority)
    {
        String token = request.getHeader("Authorization");
        try {
            logger.info(jwtUtils.getUsername(token)+" start to add an authority:"+objectMapper.writeValueAsString(authority));
            return authorityService.add(authority);
        }
        catch (Exception ex)
        {
            logger.info(jwtUtils.getUsername(token)+" start to add an authority!!!");

        }
        return null;
    }
    @RequestMapping(value = "/authorities/{id}",method = RequestMethod.DELETE)
    public Object deleteAuthority(HttpServletRequest request, @PathVariable int id)
    {
        String token = request.getHeader("Authorization");
        try {
            logger.info(jwtUtils.getUsername(token)+" start to delete an authority:"+id);
            return authorityService.deleteById(id);
        }
        catch (Exception ex)
        {
            logger.info(jwtUtils.getUsername(token)+" start to delete an authority!!!");

        }
        return null;
    }
    @RequestMapping(value =  "/authorities/{id}",method = RequestMethod.PUT)
    public Object updateAuthority(HttpServletRequest request,@PathVariable int id,@RequestBody Authority authority){
        String token = request.getHeader("Authorization");
        try {
            logger.info(jwtUtils.getUsername(token)+" start to update auth:"+id+": "+objectMapper.writeValueAsString(authority));
            return authorityService.update(authority);
        }
        catch (Exception ex){
            ex.printStackTrace();
        }
        return null;
    }

}
