package com.lstznkj.groundstation.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.lstznkj.groundstation.utils.DateUtil;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

import javax.persistence.*;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

@Entity
public class SysRole {
    @Id
    @GeneratedValue
    @Column(name = "role_id")
    private Integer id;
    @Column(unique = true)
    private String name;

    private String description;


    @CreationTimestamp
    private Timestamp createTimestamp;

    @CreationTimestamp
    private Timestamp updateTimestamp;

    @Transient
    private String createTime;
    @Transient
    private String updateTime;



    @ManyToMany(fetch = FetchType.EAGER,mappedBy = "sysRoles")
    private List<User> users=new ArrayList<>();




    //后端接口权限
    @ManyToMany(cascade = CascadeType.ALL)
    @LazyCollection(LazyCollectionOption.FALSE)
    @JoinTable(name ="sys_role_authorities", joinColumns = {@JoinColumn(name = "role_id",referencedColumnName = "role_id")},
            inverseJoinColumns = {@JoinColumn(name = "auth_id",referencedColumnName = "auth_id")})
    private List<Authority> authorities =new ArrayList<>();

    //前端路由权限,一对多。


    @JsonIgnore
    public void setUsers(List<User> users) {
        this.users = users;
    }

    @JsonIgnore
    public List<User> getUsers() {
        return users;
    }

    public Timestamp getCreateTimestamp() {
        return createTimestamp;
    }

    public Timestamp getUpdateTimestamp() {
        return updateTimestamp;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getDescription() {
        return description;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setAuthorities(List<Authority> authorities) {
        this.authorities = authorities;
    }



    public List<Authority> getAuthorities() {
        return authorities;
    }
    public String getCreateTime(){
        if(this.createTimestamp==null){
            this.createTime="";
        }else {
            this.createTime= DateUtil.getSingleInstance().formatToMillisecond(this.createTimestamp.getTime());
        }

        return this.createTime;
    }
    public String getUpdateTime(){
        if(this.updateTimestamp==null){
            this.updateTime="";
        }
        else {
            this.updateTime= DateUtil.getSingleInstance().formatToMillisecond(this.updateTimestamp.getTime());
        }

        return this.updateTime;
    }

    public void setCreateTimestamp(Timestamp createTimestamp) {
        this.createTimestamp = createTimestamp;
    }

    public void setUpdateTimestamp(Timestamp updateTimestamp) {
        this.updateTimestamp = updateTimestamp;
    }

    public void setCreateTime(String createTime) {
        this.createTime = createTime;
    }

    public void setUpdateTime(String updateTime) {
        this.updateTime = updateTime;
    }
}