package com.lstznkj.groundstation.model.crane;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.lstznkj.groundstation.model.crane.extra.*;
import com.lstznkj.groundstation.utils.DateUtil;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.UpdateTimestamp;

import javax.persistence.*;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;


//机构部位
@Entity
public class Device {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    //数据库唯一标识符
    private Integer id;

    @CreationTimestamp
    private Timestamp createTimestamp;
    @UpdateTimestamp
    private Timestamp updateTimestamp;

    @Transient
    private String createTime;
    @Transient
    private String updateTime;
    private String name;
    private String remark;

    @Transient
    private String fullName;


    //设备现有状态： 正常、报警、预警、故障……
    @ManyToOne
    private State state;


    /****
     * 工作和寿命 统计分析这块后期应单独分离
     */

    //工作运行时间
    private long workTime;
    //工作次数
    private long workCount;
    //预设寿命时长
    private long lifeTime;
    //寿命次数
    private long lifeCount;

    @Transient
    private String label;
    @Transient
    private int value;

    //额外信息,1级设备-天车存储 mqtt 服务器地址
    private String extra;

    //规格
    private String specification;


    //设备级别，1级设备、二级设备........
    private int level;


    @OneToMany(mappedBy = "father", fetch = FetchType.EAGER)
    private List<Device> children;

    @JsonIgnore
    @OneToMany(mappedBy = "device")
    private List<WorkLog> workLogList;

    @JsonIgnore
    @ManyToOne(cascade = CascadeType.DETACH)
    private Device father;

    @Transient
    private int fatherId;


    @Transient
    private int childrenCount;




    //设备改造记录
    @OneToMany(mappedBy = "device",fetch = FetchType.LAZY)
    private List<ReformLog> reformLogList;


    //设备更换记录
    @OneToMany(mappedBy = "device",fetch = FetchType.LAZY)
    private List<ExchangeLog> exchangeLogList;

    //事故 隐患
    @OneToMany(mappedBy = "device",fetch = FetchType.LAZY)
    private List<AccidentLog> accidentLogList;


    //点检标准
    @OneToOne(mappedBy = "device",fetch = FetchType.LAZY)
    @JoinColumn(name = "point_check_standard_id")
    private PointCheckStandard pointCheckStandard;




    /**
     * 一个设备有多种数据采集，比如振动、温度、
     * 每个数据类型下面有单独的数据历史记录
     */

    @OneToMany(mappedBy = "device", cascade = CascadeType.ALL)
    private List<DataSource> dataSources;

//    @OneToOne
//    private LifeModel lifeModel;






    public long getLifeTime() {
        return lifeTime;
    }


    public void setFullName(String fullName) {
        this.fullName = fullName;
    }

    private String getFullName(Device device) {
        StringBuffer nameBuffer = new StringBuffer();
        nameBuffer.append(device.getName());
        if (device.getFather() != null) {
            nameBuffer.append("/" + getFullName(device.getFather()));

            //System.out.println(device.getName()+" father is "+device.getFather().getName());
            return nameBuffer.toString();

        } else {
            //nameBuffer.append(device.getName());

            return nameBuffer.toString();
        }


    }

    public String getFullName() {

        //System.out.println("fullname : "+this.fullName);
        String[] tmp = getFullName(this).split("/");
        StringBuffer tmpBuffer = new StringBuffer();
        for (int i = tmp.length - 1; i >= 0; --i) {
            tmpBuffer.append(tmp[i] + "/");
        }
        this.fullName = tmpBuffer.toString();
        return fullName;
    }

    public void setLifeTime(long lifeTime) {
        this.lifeTime = lifeTime;
    }

    //存储所有子设备的数据源
    @Transient
    private List<DataSource> childrenDataSources = new ArrayList<>();

    public void setWorkCount(long workCount) {
        this.workCount = workCount;
    }

    public void setLifeCount(long lifeCount) {
        this.lifeCount = lifeCount;
    }

    public long getWorkCount() {
        return workCount;
    }

    public long getLifeCount() {
        return lifeCount;
    }

    public void setExtra(String extra) {
        this.extra = extra;
    }

    public String getExtra() {
        return extra;
    }


    //所有子设备
    @Transient
    private List<Device> allChildren = new ArrayList<>();

    public PointCheckStandard getPointCheckStandard() {
        return pointCheckStandard;
    }

    public void setPointCheckStandard(PointCheckStandard pointCheckStandard) {
        this.pointCheckStandard = pointCheckStandard;
    }

    private List<Device> getChildren(Device device) {
        List<Device> deviceList = new ArrayList<>();
        if (device.children != null) {
            deviceList.addAll(device.children);
            for (Device child : device.children) {
                deviceList.addAll(getChildren(child));
            }
        }
        return deviceList;
    }

    public List<Device> getAllChildren() {

        allChildren = getChildren(this);
        return allChildren;
    }

    public State getState() {
        return state;
    }

    public void setState(State state) {
        this.state = state;
    }

    public void setAllChildren(List<Device> allChildren) {
        this.allChildren = allChildren;
    }

    public void setDataSources(List<DataSource> dataSources) {
        this.dataSources = dataSources;
    }

    public void setChildrenDataSources(List<DataSource> childrenDataSources) {
        this.childrenDataSources = childrenDataSources;
    }


    public void setWorkTime(long workTime) {
        this.workTime = workTime;
    }

    public List<DataSource> getDataSources() {
        return dataSources;
    }


    private List<DataSource> getChildrenDataSources(Device device) throws JsonProcessingException {
        List<DataSource> dataSourceList = new ArrayList<>();
        if (device.children != null) {
            for (Device child : device.children) {
                if (child.dataSources != null) {
//                    System.out.println(child.getName()+" dataSources:"+new ObjectMapper().writeValueAsString(child.dataSources));
                    dataSourceList.addAll(child.getDataSources());
                }
                dataSourceList.addAll(getChildrenDataSources(child));
            }
        }
//        if(device.dataSources!=null){
//            dataSourceList.addAll(device.getDataSources());
//        }

        return dataSourceList;
    }
    public List<DataSource> getModbusDataSources(){
        List<DataSource> dataSourceList = new ArrayList<>();
        if(this.getDataSources()==null){
            return null;
        }
        for(DataSource dataSource:this.getDataSources()){
            if(dataSource.getAddress().split("/")[0].equals("485")){
                dataSourceList.add(dataSource);
            }
        }

        return dataSourceList;
    }

    /***
     * 获取所有子设备的485数据源
     * @return
     * @throws JsonProcessingException
     */
    public List<DataSource> getChildrenDataSources() throws JsonProcessingException {
        if(this.childrenDataSources!=null){
            this.childrenDataSources.clear();
        }
//        System.out.println(getName()+ ":children length "+ children.size());
        List<DataSource> dataSourceList = new ArrayList<>();
        dataSourceList.addAll(getChildrenDataSources(this));

//        if(children!=null){
//            for(Device child : children){
//                if(child.dataSources!=null){
//                    childrenDataSources.addAll(child.getDataSources());
//                }
//            }
//        }
        childrenDataSources.addAll(dataSourceList);
        return childrenDataSources;
    }
    public List<DataSource> getChildrenModbusDataSources() throws JsonProcessingException {
        List<DataSource> dataSourceList = new ArrayList<>();
        List<DataSource> tmpList=this.getChildrenDataSources();
        for(DataSource dataSource:tmpList){
            if(dataSource.getAddress().split("/")[0].equals("485")){
                dataSourceList.add(dataSource);
            }
        }
        return dataSourceList;
    }

    public long getWorkTime() {
        return workTime;
    }

    public List<WorkLog> getWorkLogList() {
        return workLogList;
    }

    public void setWorkLogList(List<WorkLog> workLogList) {
        this.workLogList = workLogList;
    }

    public int getFatherId() {
        if (father != null) {
            fatherId = father.getId();
        }


        return fatherId;
    }

    public void setFatherId(int fatherId) {
        this.fatherId = fatherId;
    }

    public int getChildrenCount() {
        if (this.children == null) {
            this.childrenCount = 0;
        } else {
            this.childrenCount = this.children.size();
        }

        return this.childrenCount;
    }

    public int getValue() {
        if (id != null) {
            this.value = id;
        } else {
            this.value = 0;
        }

        return value;
    }

    public void setValue(int value) {
        this.value = value;
    }

    public void setChildrenCount(int childrenCount) {
        this.childrenCount = childrenCount;
    }

    public void setLabel(String label) {
        this.label = label;
    }


    public void setUpdateTime(String updateTime) {
        this.updateTime = updateTime;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public void setCreateTime(String createTime) {
        this.createTime = createTime;
    }


    public void setCreateTimestamp(Timestamp createTimestamp) {
        this.createTimestamp = createTimestamp;
    }

    public void setUpdateTimestamp(Timestamp updateTimestamp) {
        this.updateTimestamp = updateTimestamp;
    }


    public Timestamp getCreateTimestamp() {
        return createTimestamp;
    }

    public Timestamp getUpdateTimestamp() {
        return updateTimestamp;
    }


    public void setChildren(List<Device> children) {
        this.children = children;
    }

    public void setFather(Device father) {
        this.father = father;
    }

    public List<Device> getChildren() {
        return children;
    }

    public Device getFather() {
        return father;
    }


    public void setId(Integer id) {
        this.id = id;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setLevel(int level) {
        this.level = level;
    }

    public Integer getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public int getLevel() {
//        if(this.father!=null){
//            this.level=father.getLevel()+1;
//        }
        return level;
    }


    public String getCreateTime() {
        if (this.createTimestamp == null) {
            this.createTime = "";
        } else {
            this.createTime = DateUtil.getSingleInstance().formatToSecond(this.createTimestamp.getTime());
        }

        return this.createTime;
    }

    public String getUpdateTime() {
        if (this.updateTimestamp == null) {
            this.updateTime = "";
        } else {
            this.updateTime = DateUtil.getSingleInstance().formatToSecond(this.updateTimestamp.getTime());
        }

        return this.updateTime;
    }

    /**
     * 供前端树形控件使用
     *
     * @return
     */
    public String getLabel() {
        this.label = this.name;
        return label;
    }

    public String getSpecification() {
        return specification;
    }

    public void setSpecification(String specification) {
        this.specification = specification;
    }



    public List<ReformLog> getReformLogList() {
        return reformLogList;
    }

    public void setReformLogList(List<ReformLog> reformLogList) {
        this.reformLogList = reformLogList;
    }

    public List<ExchangeLog> getExchangeLogList() {
        return exchangeLogList;
    }

    public void setExchangeLogList(List<ExchangeLog> exchangeLogList) {
        this.exchangeLogList = exchangeLogList;
    }

    public List<AccidentLog> getAccidentLogList() {
        return accidentLogList;
    }

    public void setAccidentLogList(List<AccidentLog> accidentLogList) {
        this.accidentLogList = accidentLogList;
    }
}
