package com.lstznkj.groundstation.model.crane;

import com.lstznkj.groundstation.utils.DateUtil;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.UpdateTimestamp;

import javax.persistence.*;
import java.sql.Timestamp;

@Entity
public class WorkLog {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    //数据库唯一标识符
    private Long id;

    public WorkLog(){
        this.abnormalStop=true;
    }

    //运行记录间隔
    @Column(name = "_interva")
    private int interval;

    @CreationTimestamp
    private Timestamp createTimestamp;
    @UpdateTimestamp
    private Timestamp updateTimestamp;

    @Transient
    private String createTime;
    @Transient
    private String updateTime;

    //上次启动时间
    private Timestamp startTimestamp;
    //上次停止时间
    private Timestamp stopTimestamp;
    //异常停止记录标志，默认true, 设备每次正常的运行 结束 会置为false
    private boolean abnormalStop;
    private String startTime;
    private String stopTime;


    //启动时的额外信息
    @Lob
    private String startExtra;
    //停止时的额外信息
    @Lob
    private String stopExtra;

    @ManyToOne(cascade = CascadeType.ALL)
    private Device device;


    public String getStartTime() {
        if(this.startTimestamp==null){
            this.startTime="";
        }else {
            this.startTime= DateUtil.getSingleInstance().formatToMillisecond(this.startTimestamp.getTime());
        }

        return this.startTime;
    }

    public String getStopTime() {
        if(this.startTimestamp==null){
            this.stopTime="";
        }else {
            this.stopTime= DateUtil.getSingleInstance().formatToMillisecond(this.stopTimestamp.getTime());
        }

        return this.startTime;
    }

    public Device getDevice() {
        return device;
    }

    public void setDevice(Device device) {
        this.device = device;
    }

    public String getCreateTime(){
        if(this.createTimestamp==null){
            this.createTime="";
        }else {
            this.createTime= DateUtil.getSingleInstance().formatToMillisecond(this.createTimestamp.getTime());
        }

        return this.createTime;
    }
    public String getUpdateTime(){
        if(this.updateTimestamp==null){
            this.updateTime="";
        }
        else {
            this.updateTime= DateUtil.getSingleInstance().formatToMillisecond(this.updateTimestamp.getTime());
        }

        return this.updateTime;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public void setInterval(int interval) {
        this.interval = interval;
    }

    public void setCreateTimestamp(Timestamp createTimestamp) {
        this.createTimestamp = createTimestamp;
    }

    public void setUpdateTimestamp(Timestamp updateTimestamp) {
        this.updateTimestamp = updateTimestamp;
    }

    public void setCreateTime(String createTime) {
        this.createTime = createTime;
    }

    public void setUpdateTime(String updateTime) {
        this.updateTime = updateTime;
    }

    public void setStartTimestamp(Timestamp startTimestamp) {
        this.startTimestamp = startTimestamp;
    }

    public void setStopTimestamp(Timestamp stopTimestamp) {
        this.stopTimestamp = stopTimestamp;
    }

    public void setAbnormalStop(boolean abnormalStop) {
        this.abnormalStop = abnormalStop;
    }

    public void setStartTime(String startTime) {
        this.startTime = startTime;
    }

    public void setStopTime(String stopTime) {
        this.stopTime = stopTime;
    }

    public Long getId() {
        return id;
    }

    public int getInterval() {
        return interval;
    }

    public Timestamp getCreateTimestamp() {
        return createTimestamp;
    }

    public Timestamp getUpdateTimestamp() {
        return updateTimestamp;
    }

    public Timestamp getStartTimestamp() {
        return startTimestamp;
    }

    public Timestamp getStopTimestamp() {
        return stopTimestamp;
    }

    public boolean isAbnormalStop() {
        return abnormalStop;
    }

    public String getStartExtra() {
        return startExtra;
    }

    public void setStartExtra(String startExtra) {
        this.startExtra = startExtra;
    }

    public String getStopExtra() {
        return stopExtra;
    }

    public void setStopExtra(String stopExtra) {
        this.stopExtra = stopExtra;
    }


}
