package com.lstznkj.groundstation.security;

import com.lstznkj.groundstation.model.Authority;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.access.AccessDecisionManager;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.access.ConfigAttribute;
import org.springframework.security.authentication.InsufficientAuthenticationException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.web.FilterInvocation;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Collection;



@Service
public class UrlAccessDecisionManager implements AccessDecisionManager {
    private Logger logger= LoggerFactory.getLogger(UrlAccessDecisionManager.class);

    @Override
    public void decide(Authentication authentication, Object object, Collection<ConfigAttribute> configAttributes) throws AccessDeniedException, InsufficientAuthenticationException {
        HttpServletRequest request = ((FilterInvocation) object).getHttpRequest();
        String url, method;
        logger.info("Decision:"+authentication.getPrincipal());
        logger.info("Decision autorities:"+authentication.getAuthorities());
        if(matchers("/user/logout",request))
        {
            logger.info( "start to logout");
            return;
        }

        if(matchers("/swagger-ui.html",request))
        {
            logger.info( "swagger");
            return;
        }
        if(matchers("/devices/*/dataSources",request)&&request.getMethod().equals("GET")){
            logger.info("get crane dataSources");
            return;
        }
        if(matchers("/devices/*/dataSourcePoolDTO",request)&&request.getMethod().equals("GET")){
            logger.info("get crane dataSourcesPoolDTO");
            return;
        }
        if(matchers("/test/**",request)&&request.getMethod().equals("GET")){
            logger.info("test interface");
            return;
        }
        for (GrantedAuthority ga : authentication.getAuthorities()) {

            if (ga instanceof Authority) {
                Authority authority = (Authority) ga;
                url =authority.getApiUrl();
                method = authority.getMethod();
                if (matchers(url, request)) {
                    if (method.equals(request.getMethod()) || "ALL".equals(method)) {
                        return;
                    }
                }
            }
        }
        logger.error(authentication.getPrincipal()+" has no right to "+request);
        throw new AccessDeniedException("no right");
    }


    @Override
    public boolean supports(ConfigAttribute attribute) {
        return true;
    }

    @Override
    public boolean supports(Class<?> clazz) {
        return true;
    }


    private boolean matchers(String url, HttpServletRequest request) {
        AntPathRequestMatcher matcher = new AntPathRequestMatcher(url);
        logger.info(url +" start to match "+ request.getRequestURI()+",method:"+request.getMethod());
        if (matcher.matches(request)) {
            logger.info(url+" matched success");
            return true;
        }
        logger.info(url+" matched failed");

        return false;
    }
}
