package com.lstznkj.groundstation.service;

import com.lstznkj.groundstation.model.Result;
import com.lstznkj.groundstation.model.Authority;
import com.lstznkj.groundstation.model.SysRole;
import com.lstznkj.groundstation.repo.AuthorityRepository;
import com.lstznkj.groundstation.repo.SysRoleRepository;
import com.lstznkj.groundstation.security.jwt.JWTUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class AuthorityService {
    private Logger logger= LoggerFactory.getLogger(AuthorityService.class);
    @Autowired
    private AuthorityRepository authRepository;
    @Autowired
    private SysRoleRepository sysRoleRepository;
    @Autowired
    private UserService userService;
    @Autowired
    private ObjectMapper objectMapper;


    @Autowired
    private JWTUtils jwtUtils;

    /**
     * 通过jwt token 解析 获取用户
     * @param token
     * @return
     */
    public Result<Iterable<Authority>> findAll(String token)
    {
        Result<Iterable<Authority>> result=new Result<>();
        result.setSuccess(true);
        result.setInfo("test result");
        String username=jwtUtils.getUsername(token);
        String[] roles= jwtUtils.getUserRole(token).split(" ");
        for(String s :roles)
        {
            logger.info(username+"'s roles: "+s);
        }

        result.setData(authRepository.findAll());

        return result;
    }




    /***
     * 添加权限
     * @param authority
     * @return
     */
    public Result<Authority> add(Authority authority)
    {
        Result<Authority> result=new Result<>();
        //存储前应查询是否有重复的路由
        try {
            if(authRepository.existsByName(authority.getName()))
            {
                result.setSuccess(false);
                result.setInfo("此权限名称已经存在!!!");
                return result;
            }
            if(authRepository.existsByApiUrlAndMethod(authority.getApiUrl(),authority.getMethod()))
            {
                result.setSuccess(false);
                result.setInfo("此权限已经存在!!!");
                return result;
            }
            authority= authRepository.save(authority);
            result.setData(authority);
            result.setSuccess(true);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            result.setInfo("未知错误!!!");
            result.setSuccess(false);

        }
        return  result;
    }

    /***
     * 通过id删除权限
     * @param id
     * @return
     */
    public Result<Authority> deleteById(int id)
    {
        Result<Authority> result=new Result<>();
        try {
            if(authRepository.existsById(id))
            {
                //删除权限时 需要将关联的外键置空
                Authority authority=authRepository.findById(id);
                logger.info(authority.getName()+"'s roles:"+objectMapper.writeValueAsString(authority.getSysRoles()));

                for(SysRole sysRole:authority.getSysRoles())
                {
                    sysRole.getAuthorities().remove(authority);

                    sysRoleRepository.save(sysRole);
                }

                authority.setSysRoles(null);
                authRepository.save(authority);
                authRepository.deleteById(id);
                result.setSuccess(true);

            }
            else {
                result.setInfo("不存在此权限!!!");
                result.setSuccess(false);
                return result;
            }
        }
        catch (Exception ex ){
            ex.printStackTrace();
            result.setInfo("未知错误!!!");
            result.setSuccess(false);
        }
        return  result;
    }

    /***
     *更新权限信息
     * @param authority
     * @return
     */
    public Result<Authority> update(Authority authority){
        Result<Authority> result=new Result<>();
        try {
            if(authRepository.existsById(authority.getId()))
            {
                authority=authRepository.save(authority);
                result.setSuccess(true);
                result.setData(authority);
            }
            else {
                result.setInfo("不存在此权限!!!");
                result.setSuccess(false);
                return result;
            }
        }
        catch (Exception ex ){
            ex.printStackTrace();
            result.setInfo("未知错误!!!");
            result.setSuccess(false);
        }
        return result;
    }
}
