package com.lstznkj.groundstation.service;

import com.lstznkj.groundstation.model.Authority;
import com.lstznkj.groundstation.model.SysRole;
import com.lstznkj.groundstation.model.User;
import com.lstznkj.groundstation.model.crane.DataType;
import com.lstznkj.groundstation.model.crane.Device;
import com.lstznkj.groundstation.model.crane.DataSource;
import com.lstznkj.groundstation.model.crane.State;
import com.lstznkj.groundstation.repo.AuthorityRepository;
import com.lstznkj.groundstation.repo.SysRoleRepository;
import com.lstznkj.groundstation.repo.UserRepository;
import com.lstznkj.groundstation.repo.crane.DataTypeRepository;
import com.lstznkj.groundstation.repo.crane.DeviceRepository;
import com.lstznkj.groundstation.repo.crane.DataSourceRepository;
import com.lstznkj.groundstation.repo.crane.StateRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/***
 * 系统用户 角色 权限 初始化
 */
@Service
public class InitService {
    private Logger logger= LoggerFactory.getLogger(getClass());
    @Autowired
    private AuthorityRepository authorityRepository;
    @Autowired
    private BCryptPasswordEncoder passwordEncoder;
    @Autowired
    private SysRoleRepository sysRoleRepository;
    @Autowired
    private UserRepository userRepository;
    @Autowired
    private UserService userService;
    @Autowired
    private SysRoleService sysRoleService;

    @Autowired
    private DeviceRepository deviceRepository;
    @Autowired
    private DataSourceRepository dataSourceRepository;
    @Autowired
    private DataTypeRepository dataTypeRepository;
    @Autowired
    private StateRepository stateRepository;

    public void init(){
        initState();
        initDataTypes();
        initAuthorities();
        initSysRoles();
        initUsers();

//       initTestCraneData1();
      // initTestCraneData2();
    }
    public void initTestCraneData1(){
        Device crane =new Device();
        crane.setName("1#天车");
        crane.setExtra("192.168.56.105");
        crane.setLevel(0);

        crane=deviceRepository.save(crane);

        Device bigCar=new Device();
        bigCar.setName("大车");
        bigCar.setLevel(1);
        bigCar.setFather(crane);
        bigCar=deviceRepository.save(bigCar);

        //deviceRepository.save(bigCar);
        Device motor=new Device();
        motor.setName("1#电机");
        motor.setFather(bigCar);
        motor.setLevel(motor.getFather().getLevel()+1);




        motor= deviceRepository.save(motor);


        Device motor1=new Device();
        motor1.setName("2#电机");



        motor1=deviceRepository.save(motor1);
        motor1.setFather(motor.getFather());
        motor1.setLevel(motor1.getFather().getLevel()+1);
        motor1=deviceRepository.save(motor1);

        List<Device> devices=new ArrayList<>();
        devices.add(bigCar);

        DataSource shockDataSource=new DataSource();
        shockDataSource.setName("振动");
        shockDataSource.setAddress("485/127.0.0.1/1/0");
        shockDataSource.setPublishTopic("shock1");
        shockDataSource.setEnable(true);
        shockDataSource= dataSourceRepository.save(shockDataSource);

        shockDataSource.setDevice(motor1);
        shockDataSource.setDataType(dataTypeRepository.findByName("振动"));

        shockDataSource= dataSourceRepository.save(shockDataSource);




       // craneRepository.save(crane);

    }

    public void initTestCraneData2(){
        Device crane =new Device();
        crane.setName("2#天车");
        crane.setExtra("192.168.56.105");
        crane.setLevel(0);

        crane=deviceRepository.save(crane);

        Device bigCar=new Device();
        bigCar.setName("大车");
        bigCar.setLevel(1);
        bigCar.setFather(crane);
        bigCar=deviceRepository.save(bigCar);

        //deviceRepository.save(bigCar);
        Device motor=new Device();
        motor.setName("1#电机");
        motor.setFather(bigCar);
        motor.setLevel(motor.getFather().getLevel()+1);




        motor= deviceRepository.save(motor);


        Device motor1=new Device();
        motor1.setName("2#电机");



        motor1=deviceRepository.save(motor1);
        motor1.setFather(motor.getFather());
        motor1.setLevel(motor1.getFather().getLevel()+1);
        motor1=deviceRepository.save(motor1);

        List<Device> devices=new ArrayList<>();
        devices.add(bigCar);

        DataSource shockDataSource=new DataSource();
        shockDataSource.setName("振动");
        shockDataSource.setAddress("485/127.0.0.1/1/0");
        shockDataSource.setPublishTopic("shock222");
        shockDataSource.setEnable(true);
        shockDataSource= dataSourceRepository.save(shockDataSource);

        shockDataSource.setDevice(motor1);
        shockDataSource.setDataType(dataTypeRepository.findByName("振动"));

        shockDataSource= dataSourceRepository.save(shockDataSource);




       // craneRepository.save(crane);

    }

    public void initState(){
        State normalState=new State();
        //预警
        State warningState =new State();
        //报警
        State alertState =new State();
        //损坏 故障
        State brokenState =new State();

        normalState.setName("正常");
        warningState.setName("预警");
        alertState.setName("报警");
        brokenState.setName("故障");

        if(!stateRepository.existsByName("正常")){
            stateRepository.save(normalState);
        }
        if(!stateRepository.existsByName("预警")){
            stateRepository.save(warningState);
        }
        if(!stateRepository.existsByName("报警")){
            stateRepository.save(alertState);
        }
        if(!stateRepository.existsByName("故障")){
            stateRepository.save(brokenState);
        }


    }

    public void initAuthorities(){

        if(authorityRepository.findAll().size()==0){
            initUltimateAuthority();
            initRoleAuthority();
            initUserAuthority();
        }


    }
    public void initDataTypes(){
        if(dataTypeRepository.findAll().size()==0){
            DataType shockType=new DataType();
            shockType.setName("振动");
            shockType.setMethod(0);
            shockType.setValueType(0);
            shockType.setRemark("振动模拟量,4-20mA");
            shockType.setExpression("16/3276*x/16*20");;

            DataType tempratureType=new DataType();
            tempratureType.setName("温度");
            tempratureType.setMethod(0);
            tempratureType.setValueType(0);
            tempratureType.setRemark("温度模拟量.4-20mA");
            tempratureType.setExpression("16/3276*x/16*200-50");

            dataTypeRepository.save(shockType);
            dataTypeRepository.save(tempratureType);
        }


    }
    private void initUltimateAuthority(){
        //后端超级管理员权限 ，操作/下所有资源
        Authority ultimateAuth=new Authority();
        ultimateAuth.setApiUrl("/**");
        ultimateAuth.setName("超级管理员");
        ultimateAuth.setMethod("ALL");
        if(!authorityRepository.existsByName("超级管理员")){
            authorityRepository.save(ultimateAuth);
        }
    }
    private void initRoleAuthority(){
        //后端添加角色权限
        Authority roleAuthAdd=new Authority();
        //后端删除角色权限
        Authority roleAuthDel=new Authority();
        //后端修改角色权限
        Authority roleAuthUpd=new Authority();
        //后端查看角色权限
        Authority roleAuthGet=new Authority();

        roleAuthGet.setApiUrl("/roles/**");
        roleAuthDel.setApiUrl("/roles/**");
        roleAuthUpd.setApiUrl("/roles/**");
        roleAuthAdd.setApiUrl("/roles/**");

        roleAuthAdd.setMethod("POST");
        roleAuthUpd.setMethod("PUT");
        roleAuthDel.setMethod("DELETE");
        roleAuthGet.setMethod("GET");

        roleAuthAdd.setName("添加角色");
        roleAuthDel.setName("删除角色");
        roleAuthGet.setName("查看角色");
        roleAuthUpd.setName("修改角色");

        if(!authorityRepository.existsByName("添加角色")){
            authorityRepository.save(roleAuthAdd);
        }
        if(!authorityRepository.existsByName("查看角色")){
            authorityRepository.save(roleAuthGet);
        }
        if(!authorityRepository.existsByName("修改角色")){
            authorityRepository.save(roleAuthUpd);
        }
        if(!authorityRepository.existsByName("删除角色")){
            authorityRepository.save(roleAuthDel);
        }
    }

    /**
     * 获取用户信息权限,登录后 获取自身信息，
     * UserController 里 根据token识别，是否是获取自身的用户信息
     */
    private void initUserAuthority(){
        //获取用户信息权限
        Authority userInfoAuth=new Authority();

        userInfoAuth.setApiUrl("/users/**");
        userInfoAuth.setName("获取用户信息");
        userInfoAuth.setMethod("GET");
        if(!authorityRepository.existsByName("获取用户信息")){
            authorityRepository.save(userInfoAuth);
        }
        else {
            logger.info("获取用户信息权限已经存在!!!");
        }


    }
    public void initSysRoles(){

        initUserRole();


        initAdminRole();


        initUltimateRole();


    }

    private void initUserRole() {
        //普同用户
        SysRole userSysRole=new SysRole();


        userSysRole.setName("ROLE_USER");
        userSysRole.setDescription("普通用户");

        //获取用户信息权限
        Authority userInfoAuth=authorityRepository.findAuthorityByName("获取用户信息");

        if(!sysRoleRepository.existsSysRoleByName("ROLE_USER")){
            userSysRole=sysRoleRepository.save(userSysRole);
            userSysRole.getAuthorities().add(userInfoAuth);
            sysRoleRepository.save(userSysRole);

//            userInfoAuth.getSysRoles().add(userSysRole);
//            authorityRepository.save(userInfoAuth);
        }
        else {
            logger.info("普通用户角色已经创建!!!");
        }
    }

    private void initAdminRole() {
        //管理员
        SysRole adminSysRole=new SysRole();

        adminSysRole.setName("ROLE_ADMIN");
        adminSysRole.setDescription("管理员");

        //获取用户信息权限
        Authority userInfoAuth=authorityRepository.findAuthorityByName("获取用户信息");

        //查看、添加、删除、修改 角色-权限
        List<Authority> roleAuthorityList= authorityRepository.findAuthoritiesByApiUrl("/roles/**");


        //**********为管理员添加 查看、添加、删除、修改角色的权限********************
        if(!sysRoleRepository.existsSysRoleByName("ROLE_ADMIN")){

            adminSysRole= sysRoleRepository.save(adminSysRole);
            adminSysRole.setAuthorities(roleAuthorityList);
            adminSysRole= sysRoleRepository.save(adminSysRole);


            userInfoAuth.getSysRoles().add(adminSysRole);
            authorityRepository.save(userInfoAuth);
            adminSysRole.getAuthorities().add(userInfoAuth);
            adminSysRole= sysRoleRepository.save(adminSysRole);


            for(Authority authority:roleAuthorityList){
                authority.getSysRoles().add(adminSysRole);
                authorityRepository.save(authority);
            }

        }
        else {
            logger.info("管理员角色已经创建!!!");
        }
        //end**********为管理员添加 查看、添加、删除、修改角色的权限********************
    }

    /***
     * 初始化超级管理员角色
     *
     */

    private void initUltimateRole() {
        //超级管理员权限
        Authority ultimateAuth=authorityRepository.findAuthorityByName("超级管理员");

        //终极管理员，拥有修改删除所有信息权限
        SysRole ultimateSysRole=new SysRole();
        ultimateSysRole.setName(("ROLE_ULTIMATE"));
        ultimateSysRole.setDescription("超级管理员");

        //************创建超级管理员角色*************************************
        if(!sysRoleRepository.existsSysRoleByName("ROLE_ULTIMATE")){
            ultimateSysRole=sysRoleRepository.save(ultimateSysRole);
            ultimateAuth.getSysRoles().add(ultimateSysRole);
            authorityRepository.save(ultimateAuth);

            ultimateSysRole.getAuthorities().add(ultimateAuth);

            sysRoleRepository.save(ultimateSysRole);
        }
        else {
            logger.info("超级管理员角色已经创建!!!");
        }
        //end ************创建超级管理员角色*************************************
    }

    public void initUsers( ){

        // sys_role_users 表中 存在重复数据 ，待修复！！！！

        User adminUlt=new User();
        adminUlt.setName("ultimate");
        adminUlt.setUsername("adminult");
        adminUlt.setPassword(passwordEncoder.encode("123456"));

        SysRole adminSysRole = sysRoleService.getUltimateSysRole();

        userService.createUser(adminUlt,adminSysRole);
    }
}
