package com.lstznkj.groundstation.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.lstznkj.groundstation.model.Result;
import com.lstznkj.groundstation.model.SysRole;
import com.lstznkj.groundstation.model.User;
import com.lstznkj.groundstation.repo.SysRoleRepository;
import com.lstznkj.groundstation.repo.UserRepository;
import com.lstznkj.groundstation.utils.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

@Service
public class UserService {
    @Autowired
    private DateUtil dateUtil;
    private Logger logger= LoggerFactory.getLogger(UserService.class);
    @Autowired
    private UserRepository userRepository;
    @Autowired
    private SysRoleRepository sysRoleRepository;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private BCryptPasswordEncoder passwordEncoder;


    @Autowired
    private SysRoleService sysRoleService;


    public Result<List<User>> findAllUsers(){
        Result<List<User>> result=new Result<>();

        List<User> users=new ArrayList<>();
        try {
            for(User user : userRepository.findAll()){
                user.setPassword("");
                users.add(user);
            }
            result.setSuccess(true);
            result.setData(users);
        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
        }

        return  result;
    }

    public Result<User> findUserByAccount(String account) {
        Result<User> result=new Result<>();
        try{
            User user=userRepository.findUserByUsername(account);
            if(user!=null)
            {
                user.setPassword("");
                result.setData(user);
                result.setSuccess(true);
            }
        }
        catch (Exception ex)
        {
            result.setSuccess(false);
            result.setInfo("error!!!");
        }
        return result;

    }

    /***
     * 创建指定角色（单个角色） 的用户
     * @param user
     * @param sysRole
     * @return
     */
    public Result<User> createUser(User user, SysRole sysRole){
        Result<User> result;
        List<SysRole> sysRoleList=new ArrayList<>();
        sysRoleList.add(sysRole);
        result=createUser(user,sysRoleList);
        return result;
    }

    /***
     * 管理员修改用户密码
     * @param user
     * @return
     */
    public Result<User> adminUpdateUserPassword(User user){
        Result<User> result=new Result<>();
        try {
            if (userRepository.existsUserByUsername(user.getUsername())) {
                User originUser=userRepository.findUserByUsername(user.getUsername());
                originUser.setPassword(passwordEncoder.encode(user.getPassword()));
                //originUser.setUpdateTime(dateUtil.getDateTimeNow());
                originUser=userRepository.save(originUser);
                originUser.setPassword("");

                result.setSuccess(true);
                result.setData(originUser);
                return result;
            } else {
                result.setSuccess(false);
                result.setInfo("用户不存在!!!");
            }
        }catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }

        return result;
    }
    /***
     * 创建指定角色的用户
     * @param user
     * @param sysRoleList
     * @return

     */
    public Result<User> createUser(User user, List<SysRole> sysRoleList){
        Result<User> result=new Result<>();
        try {
            if(!userRepository.existsUserByUsername(user.getUsername())){
                user.setSysRoles(null);
                //user.setCreateTime(dateUtil.getDateTimeNow());
                user=userRepository.save(user);
              /*  for(SysRole sysRole : sysRoleList){
                    sysRole=sysRoleRepository.findSysRoleByName(sysRole.getName());
                    sysRole.getUsers().add(user);
//                    logger.info("sysRolelist:"+objectMapper.writeValueAsString(sysRoleList));
                    sysRoleRepository.save(sysRole);
                    logger.info(user.getUsername()+ " add role: "+sysRole.getName());
                }*/
                user.setSysRoles(sysRoleList);
              //  user.setCreateTime(dateUtil.getDateTimeNow());
               // user.setUpdateTime(dateUtil.getDateTimeNow());
                user=userRepository.save(user);
                result.setSuccess(true);
                result.setData(user);
            }
            else {
                result.setSuccess(false);
                result.setInfo("用户已经存在!!!");
            }
        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }


        return result;
    }

    /***
     * 删除指定账号的用户信息
     * @param username
     * @return
     */
    @Transactional
    public Result<User> deleteUser(String username){
        Result<User> result=new Result<>();
        try {
            if(userRepository.existsUserByUsername(username)){

               userRepository.deleteUserByUsername(username);
                if(!userRepository.existsUserByUsername(username)){
                    result.setSuccess(true);
                    result.setInfo("删除成功!");
                }
                else {
                    result.setSuccess(false);
                    result.setInfo("删除失败!未能成删除账号!!!");
                }

            }
            else {
                result.setSuccess(false);
                result.setInfo("删除失败!不存在此账号!!!");
            }
        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("删除失败!未知错误!!!");
        }


        return result;
    }

    /***
     * 修改用户信息，包括拥有的角色等
     * @param user
     * @return
     */
    public Result<User> updateUser(User user){
        Result<User> result=new Result<>();

        try {
            if(userRepository.existsUserByUsername(user.getUsername())){
                User originUser= userRepository.findUserByUsername(user.getUsername());

                ///originUser.setUpdateTime(dateUtil.getDateTimeNow());
                originUser.setName(user.getName());
                originUser.setLocked(user.isLocked());
                originUser.setRemark(user.getRemark());
                originUser.setSysRoles(user.getSysRoles());
                originUser=userRepository.save(originUser);
                originUser.setPassword("");
                result.setSuccess(true);
                result.setData(originUser);
            }
            else {
                result.setSuccess(false);
                result.setInfo("修改用户信息失败,不存在此用户!!");
            }

        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("修改用户信息失败,未知错误!!");
        }





        return result;

    }

    /***
     * 添加角色给指定用户
     * @param user
     * @param roleNameList
     * @return
     */
    public Result<User> addRolesOfUser(User user,List<String> roleNameList){
        Result<User> result=new Result<>();
        List<SysRole> newSysRoleList=new ArrayList<>();
        user=userRepository.findUserByUsername(user.getUsername());
        try{
            if(user!=null){
                for(String roleName: roleNameList){
                    SysRole sysRole=sysRoleRepository.findSysRoleByName(roleName);
                    newSysRoleList.add(sysRole);
                }

                List<SysRole> originSysRoleList=user.getSysRoles();

                for(SysRole newRole : newSysRoleList){
                    boolean isExist=false;

                    //***************待优化***********************
                    for(SysRole originRole:originSysRoleList){
                        if(originRole.getName()==newRole.getName()){
                            isExist=true;
                        }
                        if(!isExist){
                            originSysRoleList.add(newRole);
                        }

                    }
                    //***************待优化***********************


                    user.setSysRoles(originSysRoleList);
                    user=userRepository.save(user);

                    result.setSuccess(true);

                }

            }
            else {
                result.setSuccess(false);
                result.setInfo("用户不存在");
            }
        }
        catch (Exception ex){
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }
        return result;
    }






}
