package com.lstznkj.groundstation.service.crane;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.lstznkj.groundstation.DTO.crane_station.DataSourceDTO;
import com.lstznkj.groundstation.DTO.crane_station.DataSourcePoolDTO;
import com.lstznkj.groundstation.DTO.ui.ChartDataDTO;
import com.lstznkj.groundstation.DTO.ui.DataSourceRequestDTO;
import com.lstznkj.groundstation.DTO.ui.DataSourceUIDTO;
import com.lstznkj.groundstation.model.Result;
import com.lstznkj.groundstation.model.crane.BaseData;
import com.lstznkj.groundstation.model.crane.DataSource;
import com.lstznkj.groundstation.model.crane.DataType;
import com.lstznkj.groundstation.model.crane.Device;
import com.lstznkj.groundstation.repo.crane.BaseDataRepository;
import com.lstznkj.groundstation.repo.crane.DataSourceRepository;
import com.lstznkj.groundstation.repo.crane.DataTypeRepository;
import com.lstznkj.groundstation.repo.crane.DeviceRepository;
import com.lstznkj.groundstation.utils.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.security.PublicKey;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@Service
public class DataSourceService {
    private Logger logger= LoggerFactory.getLogger(getClass());
    @Autowired
    private DataSourceRepository dataSourceRepository;
    @Autowired
    private DataTypeRepository dataTypeRepository;
    @Autowired
    private DeviceRepository deviceRepository;
    @Autowired
    private BaseDataRepository baseDataRepository;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private DateUtil dateUtil;
    public Result<List<DataSource>> getAll(){
        Result<List<DataSource>> result=new Result<>();
        try{
            result.setData(dataSourceRepository.findAll());
            result.setSuccess(true);
        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }
        return result;
    }

    /***
     * 根据数据类型分组查询图标数据
     * @param startTime
     * @param endTime
     * @param dataSourceRequestDTOList
     * @return
     */
    public Result<List<ChartDataDTO>> getChartDataDTOList(String startTime, String endTime, List<DataSourceRequestDTO> dataSourceRequestDTOList){
        Result<List<ChartDataDTO>> result=new Result<>();
        result.setSuccess(true);


        List<ChartDataDTO> chartDataDTOList=new ArrayList<>();

        List<DataSourceUIDTO> dataSourceUIDTOList=new ArrayList<>();

        //get all dataSourceUIDTOList
        for(DataSourceRequestDTO dto:dataSourceRequestDTOList){
            Result<DataSourceUIDTO> dataSourceUIDTOResult=getDataSourceUIDTO(dto.getId(),startTime,endTime);
            if(dataSourceUIDTOResult.isSuccess()){
                dataSourceUIDTOList.add(dataSourceUIDTOResult.getData());

            }
        }

        List<List<DataSourceUIDTO>> dataSourceUIDTOGroup=new ArrayList<>();

        //start to group
        for(int i=0;i<=dataSourceUIDTOList.size();++i){

            List<DataSourceUIDTO> list=new ArrayList<>();
            DataSourceUIDTO tmp=dataSourceUIDTOList.get(0);
            list.add(tmp);

           // dataSourceUIDTOList.remove(i);
            for(int j=1;j<dataSourceUIDTOList.size();++j){
                boolean isSame= tmp.getDataType().getName().equals(dataSourceUIDTOList.get(j).getDataType().getName());

                if(isSame){
                    list.add(dataSourceUIDTOList.get(j));
                    //dataSourceUIDTOList.remove(j);
                }
                else {
                }
            }
            for(int k=0;k<list.size();++k){

                dataSourceUIDTOList.remove(list.get(k));
            }
            dataSourceUIDTOGroup.add(list);
        }


        try {
            logger.info("dataSourceUIDTOGroup size:"+dataSourceUIDTOGroup.size());
            for(int i=0;i<dataSourceUIDTOGroup.size();++i){
                ChartDataDTO chartDataDTO=new ChartDataDTO();
                chartDataDTO.setDataSourceUIDTOList(dataSourceUIDTOGroup.get(i));
                chartDataDTOList.add(chartDataDTO);
            }
            result.setData(chartDataDTOList);

        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
        }



        return  result;
    }

    public Result<ChartDataDTO> getChartDataDTO(String startTime, String endTime, List<DataSourceRequestDTO> dataSourceRequestDTOList){
        Result<ChartDataDTO> result=new Result<>();
        List<DataSourceUIDTO> dataSourceUIDTOList=new ArrayList<>();
        ChartDataDTO chartDataDTO=new ChartDataDTO();
        for(DataSourceRequestDTO dto:dataSourceRequestDTOList){
            Result<DataSourceUIDTO> dataSourceUIDTOResult=getDataSourceUIDTO(dto.getId(),startTime,endTime);

            if(dataSourceUIDTOResult.isSuccess()){
                dataSourceUIDTOList.add(dataSourceUIDTOResult.getData());
            }
        }
        chartDataDTO.setDataSourceUIDTOList(dataSourceUIDTOList);
        result.setData(chartDataDTO);
        result.setSuccess(true);


        return  result;
    }
    public Result<ChartDataDTO> getChartDataDTO(int sourceId, String startTime, String endTime){
        Result<ChartDataDTO> result=new Result<>();

        Result<DataSourceUIDTO> dataSourceUIDTOResult=getDataSourceUIDTO(sourceId,startTime,endTime);
        if(dataSourceUIDTOResult.isSuccess()){
            ChartDataDTO chartDataDTO=new ChartDataDTO();
            List<DataSourceUIDTO> dataSourceUIDTOList=new ArrayList<>();
            dataSourceUIDTOList.add(dataSourceUIDTOResult.getData());
            chartDataDTO.setDataSourceUIDTOList(dataSourceUIDTOList);
            result.setData(chartDataDTO);
            result.setSuccess(true);
        }


        return  result;
    }

    /***
     * 获取指定id的数据源 ，通过开始时间和结束时间筛选数据源下的基础检测历史数据
     * @param sourceId
     * @param startTime
     * @param endTime
     * @return
     */
    public Result<DataSourceUIDTO> getDataSourceUIDTO(int sourceId, String startTime, String endTime){
        Result<DataSourceUIDTO> result=new Result<>();
        try{

            DataSource dataSource =dataSourceRepository.findById(sourceId).get();
            Timestamp startTimestamp=new Timestamp(dateUtil.parse(startTime).getTime());
            Timestamp endTimestamp=new Timestamp(dateUtil.parse(endTime).getTime());

            Date selectStartTime=dateUtil.getDate();
            logger.info("select baseDataList start time:"+dateUtil.formatToMillisecond(selectStartTime.getTime()));
            List<BaseData> baseDataList=baseDataRepository.findByDataSourceAndCreateTimestampBetween(dataSource,startTimestamp,endTimestamp);

            Date selectEndTime=dateUtil.getDate();
            logger.info("select baseDataList:"+baseDataList.size()+ " end time:"+dateUtil.formatToMillisecond(selectEndTime.getTime()));

            logger.info("select baseDataList:"+baseDataList.size()+ " used time:"+(selectEndTime.getTime()-selectStartTime.getTime()));


            dataSource.setBaseDataList(baseDataList);
            DataSourceUIDTO dataSourceUIDTO=new DataSourceUIDTO(dataSource);
            result.setData(dataSourceUIDTO);

            result.setSuccess(true);
        }
        catch (Exception ex){
            logger.error("get datasource  history baseDataList error : "+ex.getMessage());
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");

        }
        return result;
    }

    /***
     * 添加数据源
     * @param dataSource
     * @return
     */
    public Result<DataSource> add(DataSource dataSource){
        Result<DataSource> result=new Result<>();

        try {
            if(!dataTypeRepository.existsById(dataSource.getDataTypeId())){
                result.setSuccess(false);
                result.setInfo("数据类型错误!!!");
                return result;
            }
            if(!deviceRepository.existsById(dataSource.getDeviceId())){
                result.setSuccess(false);
                result.setInfo("设备信息错误!!!");
                return result;
            }
            DataType dataType =dataTypeRepository.findById(dataSource.getDataTypeId()).get();
            Device device=deviceRepository.findDeviceById(dataSource.getDeviceId());

/*
            if(dataType.getValueType()==1){
                //开关量时不进行优化稀疏存储，天车站自动优化发布逻辑，只有数值变动时才发布
                dataSource.setSparseCount(0);

                dataSource.setDifferSparseValue(-1);
                dataSource.setTotalSparseValue(0);
            }*/

            dataSource.setSparseCount(3000);

            dataSource.setDifferSparseValue(1);

            dataSource.setTotalSparseValue(10);


            dataSource=dataSourceRepository.save(dataSource);
            dataSource.setPublishTopic(autoGenerateTopic(dataSource));
            dataSource.setDevice(device);
            dataSource.setDataType(dataType);
            dataSource=dataSourceRepository.save(dataSource);
            result.setSuccess(true);

            result.setData(dataSource);


        }catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }

        return  result;
    }

    public Result<DataSourceUIDTO> update(DataSourceUIDTO dataSourceUIDTO){
        Result<DataSourceUIDTO> result=new Result<>();


        try {
            if(dataSourceUIDTO.getDeviceId()==dataSourceUIDTO.getOldDeviceId()){
                logger.info("数据源所属机构未修改");
                DataSource originDataSource=dataSourceRepository.findById(dataSourceUIDTO.getId()).get();

                originDataSource.setName(dataSourceUIDTO.getName());
                originDataSource.setTotalSparseValue(dataSourceUIDTO.getTotalSparseValue());
                originDataSource.setMinValue(dataSourceUIDTO.getMinValue());
                originDataSource.setMaxValue(dataSourceUIDTO.getMaxValue());
                originDataSource.setThreshold(dataSourceUIDTO.getThreshold());
                originDataSource.setAddress(dataSourceUIDTO.getAddress());

                originDataSource=dataSourceRepository.save(originDataSource);
                result.setData(dataSourceUIDTO);
                result.setSuccess(true);
            }
            else {
                logger.info("数据源所属机构被修改");
                Device nDevice=deviceRepository.findDeviceById(dataSourceUIDTO.getDeviceId());
                Device oldDevice=deviceRepository.findDeviceById(dataSourceUIDTO.getOldDeviceId());

                DataSource tmpDataSource=null;
                for(DataSource dataSource:oldDevice.getDataSources()){
                    if(dataSource.getId()==dataSourceUIDTO.getId()){
                       tmpDataSource=dataSource;
                    }
                }
                oldDevice.getDataSources().remove(tmpDataSource);
                oldDevice=deviceRepository.save(oldDevice);

                DataSource originDataSource=dataSourceRepository.findById(dataSourceUIDTO.getId()).get();

                originDataSource.setName(dataSourceUIDTO.getName());
                originDataSource.setTotalSparseValue(dataSourceUIDTO.getTotalSparseValue());
                originDataSource.setMinValue(dataSourceUIDTO.getMinValue());
                originDataSource.setMaxValue(dataSourceUIDTO.getMaxValue());
                originDataSource.setAddress(dataSourceUIDTO.getAddress());

                originDataSource.setDevice(nDevice);
                nDevice.getDataSources().add(originDataSource);
                nDevice=deviceRepository.save(nDevice);
                originDataSource=dataSourceRepository.save(originDataSource);

                result.setData(dataSourceUIDTO);
                result.setSuccess(true);


            }

        }
        catch (Exception ex){
            ex.printStackTrace();
        }


        return  result;
    }

    /**
     * 获取指定id设备的所有数据源（包括子设备）
     * @param id
     * @return
     */
    public Result<List<DataSourceUIDTO>> getAllDataSourcesByDeviceId(int id){
        try {

            Result<List<DataSourceUIDTO>>  result=new Result<>();
            Device device=deviceRepository.findDeviceById(id);
            List<DataSourceUIDTO> dataSourceUIDTOList=new ArrayList<>();
            List<DataSource> childrenDataSourceList = device.getChildrenDataSources();
            logger.info( "childrenDataSources size is:"+ childrenDataSourceList.size());

            for(DataSource dataSource: childrenDataSourceList){

                DataSourceUIDTO dto=new DataSourceUIDTO(dataSource);

                dataSourceUIDTOList.add(dto);
            }
            result.setData(dataSourceUIDTOList);
            result.setSuccess(true);
            logger.info( "dataSourceUIDTOList :"+objectMapper.writeValueAsString(result));

            return result;
        }
        catch (Exception ex){
            ex.printStackTrace();
        }

        return null;
    }

    private String autoGenerateTopic(DataSource dataSource ){
//        Date date=new Date();
//        StringBuffer stringBuffer=new StringBuffer();
//        stringBuffer.append(String.valueOf(date.getTime()));
//        stringBuffer.append(dataSource.getId());
//        return stringBuffer.toString();

       return UUID.randomUUID().toString();


    }
    public Result<DataSource> deleteById(int id){
        Result<DataSource> result=new Result<>();
        try {
            dataSourceRepository.deleteById(id);
            if(!dataSourceRepository.existsById(id)){
                result.setSuccess(true);
                logger.info("delete datasource:"+id+" success!!!");
            }
            else{
                logger.error("delete datasource:"+id+" failed!!!");
                result.setSuccess(false);
            }
           
        }catch (Exception ex){
            logger.error("delete datasource:"+id+" error!!!");
            ex.printStackTrace();
            
              
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }
        return result;
    }
    /*
     * 获取指定设备的数据源集合
     */
    public Result<List<DataSource>> getDataSources(int id){
        Result<List<DataSource>> result=new Result<>();
        try {
            Device device=deviceRepository.findDeviceById(id);
            List<DataSource> dataSourceList=dataSourceRepository.findAllByDevice(device);

           // logger.info("device:"+id +" datasources size:"+dataSourceList.size());
            result.setData(dataSourceList);

        }catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }
        return result;
    }


    /***
     * 获取指定设备的所有plc数据源
     * @param id
     * @return
     */
    public Result<List<DataSource>> getAllPLCDataSources(int id){
        Result<List<DataSource>> result=new Result<>();
        List<DataSource> plcDataSourceList=new ArrayList<>();
        try {
            List<DataSource> dataSourceList=getAllDataSources(id).getData();
            for(DataSource dataSource:dataSourceList){
                String protocol=dataSource.getAddress().split("/")[0];
                if(protocol.toLowerCase().equals("plc")){
                    plcDataSourceList.add(dataSource);
                }
            }

            result.setData(plcDataSourceList);
            result.setSuccess(true);
        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
        }

        return result;
    }

    /***
     *获取指定起重机下指定名称的数据源
     * @param
     * @return
     */
    public Result<DataSource> getDataSourceByName(int craneId,String name){
        Result<DataSource> result=new Result<>();

        try {
            List<DataSource> dataSourceList=getAllDataSources(craneId).getData();
            for(DataSource dataSource:dataSourceList){
                if(dataSource.getName().equals(name)){
                    logger.info("finded dataSource for:"+name+","+objectMapper.writeValueAsString(dataSource));
                    result.setData(dataSource);
                    result.setSuccess(true);
                    return result;
                }
            }
            logger.error("can't find dataSource :"+name);


        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
        }

        return result;
    }

    /***
     * 获取指定设备的所有数据源，包括子设备
     * @param id
     * @return
     */
    public Result<List<DataSource>> getAllDataSources(int id){
        Result<List<DataSource>> result=new Result<>();
        try {
            List<DataSource> dataSourceList=new ArrayList<>();
            Device device=deviceRepository.findDeviceById(id);

            List<Device> allChildren= device.getAllChildren();
           // logger.info(device.getName()+" children size is "+allChildren.size());
            for(Device d:allChildren){

                //dataSourceList.addAll(d.getDataSources());
                dataSourceList.addAll(getDataSources(d.getId()).getData());

            }
            //dataSourceList.addAll(device.getDataSources());
            dataSourceList.addAll(getDataSources(device.getId()).getData());

            //设置每一个数据源的最后一个BaseData数据 ，用以辅助稀疏存储
            for(DataSource dataSource:dataSourceList){
                BaseData lastBaseData=baseDataRepository.findFirstByDataSourceOrderByIdDesc(dataSource);
                if(lastBaseData!=null){
                    dataSource.setLastBaseData(lastBaseData);
                }
                else {
                    lastBaseData=new BaseData();
                    dataSource.setLastBaseData(lastBaseData);
                }

            }

            result.setData(dataSourceList);
            result.setSuccess(true);
        }catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }
        return result;
    }

    public Result<List<DataSourcePoolDTO>> getDataSourcePoolDTO(int id){

        Result<List<DataSource>> resultList=new Result<>();
        Result<List<DataSourcePoolDTO>> resultDataSourcePoolDTO=new Result<>();
        List<DataSourcePoolDTO> dataSourcePoolDTOList=new ArrayList<>();

        //  " {proto}/ip/{slaveId}/{offset/{quantity}}}"
        //  “ 485/192.168.0.111/1/0/1

        //  " {proto}/ip/{rackId}/{slotId}/{area}/{address}/offset/quantity/
        //  " PLC/192.168.0.111/0/1/QW/3/

        // PLC/192.168.8.103/0/1/DB/8/12.6/1
        

        try {
            resultList=getAllDataSources(id);
            if(resultList.getData().size()>0){
                String[] firstStr =resultList.getData().get(0).getAddress().split("/");
                DataSourcePoolDTO firstDataSourcePoolDTO=new DataSourcePoolDTO();
                String firstProtoType= firstStr[0].trim();
                String  firstIpAddress= firstStr[1].trim();
                firstDataSourcePoolDTO.setProtocolType(firstProtoType);
                firstDataSourcePoolDTO.setAddress(firstIpAddress);
                dataSourcePoolDTOList.add(firstDataSourcePoolDTO);

                //筛选重复数据源地址的pool

                for(int i=0;i<resultList.getData().size();++i){
                    String[] addressStr=resultList.getData().get(i).getAddress().split("/");
                    String protoType=addressStr[0];
                    String ipAddress=addressStr[1];
                    boolean isSameProtoType=false;
                    boolean isSameIpAddress=false;
                    for(int j=0;j<dataSourcePoolDTOList.size();++j){
                        String tmpAddress=dataSourcePoolDTOList.get(j).getAddress().trim();
                        String tmpProtoType=dataSourcePoolDTOList.get(j).getProtocolType().trim();
                        if(protoType.trim().equals(tmpProtoType)){
                            isSameProtoType=true;
                        }
                        if(ipAddress.trim().equals(tmpAddress)){
                            isSameIpAddress=true;
                        }
                    }
                    if(!isSameIpAddress||!isSameProtoType){

                        DataSourcePoolDTO dataSourcePoolDTO=new DataSourcePoolDTO();
                        dataSourcePoolDTO.setProtocolType(protoType);
                        dataSourcePoolDTO.setAddress(ipAddress);
                        dataSourcePoolDTOList.add(dataSourcePoolDTO);

                    }



                }
                //end 筛选重复数据源地址的pool
                resultDataSourcePoolDTO.setData(dataSourcePoolDTOList);

                logger.info("pool:"+objectMapper.writeValueAsString(resultDataSourcePoolDTO.getData()));

            }

            for(DataSourcePoolDTO poolDTO: resultDataSourcePoolDTO.getData()){
                addDataSourcesToDataSourcePoolDTO(resultList.getData(),poolDTO);
            }

        }
        catch (Exception ex){
            ex.printStackTrace();
        }

        return  resultDataSourcePoolDTO;
    }


    /***
     * 将对应 地址和协议的 数据源 添加到数据源pool中
     * @param dataSourceList
     * @param dataSourcePoolDTO
     */
    private DataSourcePoolDTO addDataSourcesToDataSourcePoolDTO(List<DataSource> dataSourceList, DataSourcePoolDTO dataSourcePoolDTO) {
        //将相同协议和地址的数据源 添加到对应的pool中哦
            for(DataSource dataSource: dataSourceList){
                String[] addressStr=dataSource.getAddress().split("/");
                String tmpAddress=addressStr[1].trim();
                String tmpProtoType=addressStr[0].trim();

                boolean isSameProtoType=dataSourcePoolDTO.getProtocolType().trim().equals(tmpProtoType);
                boolean isSameIpAddress=dataSourcePoolDTO.getAddress().equals(tmpAddress);

                if(isSameIpAddress&&isSameProtoType){
                    DataSourceDTO dataSourceDTO=dataSource2DataSourceDTO(dataSource);

                    dataSourcePoolDTO.getDataSourceDTOList().add(dataSourceDTO);

            }
        }
            return dataSourcePoolDTO;
    }

    private DataSourceDTO dataSource2DataSourceDTO(DataSource dataSource){

        DataSourceDTO dataSourceDTO=new DataSourceDTO();
        dataSourceDTO.setId(dataSource.getId());
        dataSourceDTO.setDataType(dataSource.getDataType());
        dataSourceDTO.setName(dataSource.getName());
        dataSourceDTO.setCreateTime(dataSource.getCreateTime());
        dataSourceDTO.setCreateTimestamp(dataSource.getCreateTimestamp());
        dataSourceDTO.setUpdateTime(dataSource.getUpdateTime());
        dataSourceDTO.setUpdateTimestamp(dataSource.getUpdateTimestamp());
        dataSourceDTO.setPublishTopic(dataSource.getPublishTopic());
        dataSourceDTO.setMaxValue(dataSource.getMaxValue());
        dataSourceDTO.setMinValue(dataSource.getMinValue());
        dataSourceDTO.setThreshold(dataSource.getThreshold());

        String[] tmpAddress=dataSource.getAddress().split("/");
        /*for(int i=0;i<tmpAddress.length;++i){
            logger.info(i+":"+tmpAddress[i]);
        }*/
        dataSourceDTO.setOffset(Integer.valueOf(tmpAddress[3]));

        //dataSourceDTO.setQuantity(Integer.valueOf(tmpAddress[4]));
        dataSourceDTO.setExtraInfo(dataSource.getAddress());
        dataSourceDTO.setAddress(tmpAddress[1]);

        return dataSourceDTO;
    }
}
