package com.lstznkj.groundstation.service.crane;

import com.lstznkj.groundstation.DTO.crane_station.DataSourceDTO;
import com.lstznkj.groundstation.DTO.crane_station.DataTypeDTO;
import com.lstznkj.groundstation.model.Result;
import com.lstznkj.groundstation.model.crane.DataSource;
import com.lstznkj.groundstation.model.crane.DataType;
import com.lstznkj.groundstation.model.crane.Device;
import com.lstznkj.groundstation.repo.crane.DataTypeRepository;
import com.lstznkj.groundstation.repo.crane.DeviceRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class DataTypeService {
    @Autowired
    private DataTypeRepository dataTypeRepository;
    private Logger logger= LoggerFactory.getLogger(getClass());
    @Autowired
    private DeviceRepository deviceRepository;

    public Result<List<DataType>> getAll(){
        Result<List<DataType>> result=new Result<>();
        try{
            result.setData(dataTypeRepository.findAll());
            result.setSuccess(true);
        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }

        return result;
    }
    public Result<DataType> update(DataType dataType){
        Result<DataType> result=new Result<>();
        try{

            DataType originDataType=dataTypeRepository.findDataTypeById(dataType.getId());
            originDataType.setName(dataType.getName());
            originDataType.setValueType(dataType.getValueType());
            originDataType.setMethod(dataType.getMethod());
            originDataType.setExpression(dataType.getExpression());
            originDataType.setRemark(dataType.getRemark());
            originDataType.setUnit(dataType.getUnit());
            originDataType=dataTypeRepository.save(originDataType);
            result.setData(originDataType);

            result.setSuccess(true);
        }catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }
        return result;

    }
    public Result<DataType> add(DataType dataType){
        Result<DataType> result=new Result<>();
        try{
            result.setData(dataTypeRepository.save(dataType));
            result.setSuccess(true);
        }catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }
        return result;

    }
    private Device getRootFather(Device device){
        if(device.getFather()!=null){
            return  getRootFather(device.getFather());
        }
        return device;
    }
    public Result<List<DataTypeDTO>> findDataTypesByCraneId(int id){
        logger.info("start to get crane:"+id+" 's dataTypes(dataSources)");
        Result<List<DataTypeDTO>> result=new Result<>();
        List<DataTypeDTO> dataTypeDTOList=new ArrayList<>();
        try {
            //Device crane=deviceRepository.findDeviceById(id);
            List<DataType> dataTypeList=dataTypeRepository.findAll();

            for(DataType dataType:dataTypeList){
                List<DataSource> craneSources=new ArrayList<>();
              for(DataSource dataSource:dataType.getDataSources()){
                  Device rootFather=getRootFather(dataSource.getDevice());
                  if(rootFather.getId()==id){
                      logger.info(dataSource.getDeviceInfo()+dataSource.getName()+" is "+rootFather.getName() +"'s dataSource");
                      craneSources.add(dataSource);
                  }
                  else {
                      logger.info(dataSource.getDeviceInfo()+dataSource.getName()+" is not crane:"+id+" 's dataSource");
                  }
              }

              dataType.setDataSources(craneSources);
              DataTypeDTO dataTypeDTO=new DataTypeDTO(dataType);
              dataTypeDTOList.add(dataTypeDTO);
            }
            result.setSuccess(true);
            result.setData(dataTypeDTOList);
        }
        catch (Exception ex){
            ex.printStackTrace();
            logger.error("get crane:"+id+" 's dataTypes(dataSources) error:"+ex.getMessage());
            result.setSuccess(false);
        }

         return result;
    }
    public Result<DataType> deleteById(int id){
        Result<DataType> result=new Result<>();
        try{
            if(dataTypeRepository.existsById(id)){
                dataTypeRepository.deleteById(id);
                result.setSuccess(true);
            }
            else {
                result.setSuccess(false);
                result.setInfo("不存在此数据类型!!!");
            }
        }catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }
        return result;

    }

}
